from nameko.dependency_providers import Config
from nameko.rpc import rpc
from nameko.web.handlers import http
from nameko_prometheus import PrometheusMetrics
from nameko_sentry import SentryReporter
from nameko_zipkin import Zipkin
from werkzeug.wrappers import Request, Response

from .dependencies import SentryLoggerConfig


class Service:
    """
    Base class for nameko services.
    """

    name = "no name"

    SentryLoggerConfig()
    sentry = SentryReporter()
    config = Config()
    zipkin = Zipkin()
    metrics = PrometheusMetrics()

    @rpc
    def say_hello(self) -> str:
        """
        RPC method to ping the service to check if it can be reached.
        """
        return f"Hello from {self.name}!"

    @http("GET", "/metrics")
    def serve_metrics(self, request: Request) -> Response:
        """
        Exposes Prometheus metrics over HTTP.
        """
        return self.metrics.expose_metrics(request)



import logging
import os

from nameko.containers import WorkerContext
from nameko.extensions import DependencyProvider
from pyrabbit.api import Client
from raven import Client as RavenClient
from raven.conf import setup_logging
from raven.handlers.logging import SentryHandler

from .discovery import ServiceDiscovery


class SentryLoggerConfig(DependencyProvider):
    def setup(self):
        sentry_config = self.container.config.get("SENTRY", {})
        dsn = sentry_config.get("DSN", None)
        if dsn:
            client = RavenClient(
                dsn, environment=os.environ.get("SENTRY_ENVIRONMENT", "local")
            )
            handler = SentryHandler(client)
            handler.setLevel(logging.ERROR)
            setup_logging(handler)


class ServiceDiscoveryProvider(DependencyProvider):
    def __init__(self, management_host: str, username: str, password: str):
        self.management_host = management_host
        self.username = username
        self.password = password

    def setup(self):
        self.client = Client(self.management_host, self.username, self.password)
        self.discovery = ServiceDiscovery(self.client)

    def get_dependency(self, worker_ctx: WorkerContext) -> ServiceDiscovery:
        return self.discovery



from unittest.mock import create_autospec

from nameko.testing.services import worker_factory
from nameko_prometheus.dependencies import MetricsServer
from werkzeug.wrappers import Request

from nameko_chassis.service import Service


class MyService(Service):
    name = "my_service"


def test_say_hello():
    service = worker_factory(MyService)
    assert service.say_hello() == "Hello from my_service!"


def test_serve_metrics():
    request = create_autospec(Request)
    # replace metrics mock with a real dependency from nameko-prometheus
    service = worker_factory(MyService, metrics=MetricsServer())
    response = service.serve_metrics(request)
    assert response.status_code == 200
